#!/bin/bash

# JackTheStripper v2.0
# Deployer for Ubuntu Server 14.04 LTS
# 
# @license      http://www.gnu.org/licenses/gpl.txt  GNU GPL 3.0
# @author       Eugenia Bahit <ebahit@linux.com>
# @link         http://www.eugeniabahit.com/proyectos/jackthestripper
# @contributor  Jason Soto <jason_soto@jsitech.com>
# @link         http://www.jsitech.com/jackthestripper

source helpers.sh


# 0. Verificar si es usuario root o no 
function is_root_user() {
    if [ "$USER" != "root" ]; then
        echo "Permiso denegado."
        echo "Este programa solo puede ser ejecutado por el usuario root"
        exit
    else
        clear
        cat templates/texts/welcome
    fi
}


# 1. Configurar Hostname
function set_hostname() {
    write_title "1. Configurar Hostname"
    echo -n " ¿Desea configurar un hostname? (y/n): "; read config_host
    if [ "$config_host" == "y" ]; then
        serverip=$(__get_ip)
        echo " Ingrese un nombre para identificar a este servidor"
        echo -n " (por ejemplo: myserver) "; read host_name
        echo -n " ¿Cuál será el dominio principal? "; read domain_name
        echo $host_name > /etc/hostname
        hostname -F /etc/hostname
        echo "127.0.0.1    localhost.localdomain      localhost" >> /etc/hosts
        echo "$serverip    $host_name.$domain_name    $host_name" >> /etc/hosts
    fi
    say_done
}


# 2. Configurar zona horaria
function set_hour() {
    write_title "2. Configuración de la zona horaria"
    dpkg-reconfigure tzdata
    say_done
}


#  3. Actualizar el sistema
function sysupdate() {
    write_title "3. Actualización del sistema"
    apt-get update
    apt-get upgrade -y
    say_done
}


#  4. Crear un nuevo usuario con privilegios
function set_new_user() {
    write_title "4. Creación de un nuevo usuario"
    echo -n " Indique un nombre para el nuevo usuario: "; read username
    adduser $username
    usermod -a -G sudo $username
    say_done
}


#  5. Instrucciones para generar una RSA Key
function give_instructions() {
    serverip=$(__get_ip)
    write_title "5. Generación de llave RSA en su ordenador local"
    echo " *** SI NO TIENE UNA LLAVE RSA PÚBLICA EN SU ORDENADOR, GENERE UNA ***"
    echo "     Siga las instrucciones y pulse INTRO cada vez que termine una"
    echo "     tarea para recibir una nueva instrucción"
    echo " "
    echo "     EJECUTE LOS SIGUIENTES COMANDOS:"
    echo -n "     a) ssh-keygen -t rsa -b 4096 "; read foo1
    echo -n "     b) cat /home/$username/.ssh/id_rsa.pub >> /home/$username/.ssh/authorized_keys: "; read foo2
    say_done
}


#  6. Mover la llave pública RSA generada
function move_rsa() {
    write_title "6. Se moverá la llave pública RSA generada en el paso 5"
    echo " Ejecute el comando a Continuación para copiar la llave"
    echo " Presione ENTER cuando haya Finalizado "
    echo " ssh-copy-id -i $HOME/.ssh/id_rsa.pub $username@$serverip "
    say_done
}


#  7. Securizar SSH
function ssh_reconfigure() {
    write_title "7. Securizar accesos SSH"
    sed s/USERNAME/$username/g templates/sshd_config > /etc/ssh/sshd_config
    service ssh restart
    say_done
}


#  8. Establecer reglas para iptables
function set_iptables_rules() {
    write_title "8. Establecer reglas para iptables (firewall)"
    sh templates/iptables.sh
    cp templates/iptables.sh /etc/init.d/
    ln -s /etc/init.d/iptables.sh /etc/rc2.d/S99iptables.sh
    say_done
}


# 9. Instalar fail2ban
function install_fail2ban() {
    # para eliminar una regla de fail2ban en iptables utilizar:
    # iptables -D fail2ban-ssh -s IP -j DROP
    write_title "9. Instalar Sendmail y fail2ban"
    apt-get install sendmail
    apt-get install fail2ban
    say_done
}


# 10. Instalar, Configurar y Optimizar MySQL
function install_mysql() {
    write_title "10. Instalar MySQL"
    apt-get install mysql-server
    echo -n " configurando MySQL............ "
    cp templates/mysql /etc/mysql/my.cnf; echo " OK"
    mysql_secure_installation
    service mysql restart
    say_done
}


# 11. Instalar, configurar y optimizar PHP
function install_php() {
    write_title "11. Instalar PHP 5 + Apache 2"
    apt-get install apache2
    apt-get install php5 php5-cli php-pear
    apt-get install php5-mysql python-mysqldb
    echo -n " reemplazando archivo de configuración php.ini..."
    cp templates/php /etc/php5/apache2/php.ini; echo " OK"
    cp templates/php /etc/php5/cli/php.ini; echo " OK"
    service apache2 restart
    say_done
}


# 12. Instalar ModSecurity
function install_modsecurity() {
    write_title "12. Instalar ModSecurity"
    apt-get install libxml2 libxml2-dev libxml2-utils
    apt-get install libaprutil1 libaprutil1-dev
    apt-get install libapache2-mod-security2
    service apache2 restart
    say_done
}


# 13. Configurar OWASP para ModSecuity
function install_owasp_core_rule_set() {
    write_title "13. Instalar OWASP ModSecurity Core Rule Set"

    for archivo in /usr/share/modsecurity-crs/base_rules/*
        do ln -s $archivo /usr/share/modsecurity-crs/activated_rules/
    done

    for archivo in /usr/share/modsecurity-crs/optional_rules/*
        do ln -s $archivo /usr/share/modsecurity-crs/activated_rules/
    done
    echo "OK"

    sed s/SecRuleEngine\ DetectionOnly/SecRuleEngine\ On/g /etc/modsecurity/modsecurity.conf-recommended > salida
    mv salida /etc/modsecurity/modsecurity.conf
    
    echo 'SecServerSignature "AntiChino Server 1.0.4 LS"' >> /usr/share/modsecurity-crs/modsecurity_crs_10_setup.conf
    echo 'Header set X-Powered-By "Plankalkül 1.0"' >> /usr/share/modsecurity-crs/modsecurity_crs_10_setup.conf
    echo 'Header set X-Mamma "Mama mia let me go"' >> /usr/share/modsecurity-crs/modsecurity_crs_10_setup.conf

    a2enmod headers
    service apache2 restart
    say_done
}


# 14. Configurar y optimizar Apache
function configure_apache() {
    write_title "14. Finalizar configuración y optimización de Apache"
    cp templates/apache /etc/apache2/apache2.conf
    echo " -- habilitar ModRewrite"
    a2enmod rewrite
    service apache2 restart
    say_done
}


# 15. Instalar ModEvasive
function install_modevasive() {
    write_title "15. Instalar ModEvasive"
    echo -n " Indique e-mail para recibir alertas: "; read inbox
    apt-get install libapache2-mod-evasive
    mkdir /var/log/mod_evasive
    chown www-data:www-data /var/log/mod_evasive/
    sed s/MAILTO/$inbox/g templates/mod-evasive > /etc/apache2/mods-available/mod-evasive.conf
    service apache2 restart
    say_done
}

# 16. Instalar Mod_qos/spamhaus
function install_modqosspam() {
    write_title "16. Instalar Mod_qos/spamhaus"
    apt-get -y install libapache2-mod-qos
    cp templates/qos /etc/apache2/mods-available/qos.conf
    apt-get -y install libapache2-mod-spamhaus
    cp templates/spamhaus /etc/apache2/mods-available/spamhaus.conf
    service apache2 restart
    say_done
}

# 17. Configurar fail2ban
function config_fail2ban() {
    write_title "17. Finalizar configuración de fail2ban"
    sed s/MAILTO/$inbox/g templates/fail2ban > /etc/fail2ban/jail.local
    cp /etc/fail2ban/jail.local /etc/fail2ban/jail.conf
    /etc/init.d/fail2ban restart
    say_done
}


# 18. Instalación de paquetes adicionales
function install_aditional_packages() {
    write_title "18. Instalación de paquetes adicionales"
    echo "18.1. Instalar Bazaar..........."; apt-get install bzr
    echo "18.2. Instalar tree............."; apt-get install tree
    echo "18.3. Instalar Python-MySQLdb..."; apt-get install python-mysqldb
    echo "18.4. Instalar WSGI............."; apt-get install libapache2-mod-wsgi
    echo "18.5. Instalar PIP.............."; apt-get install python-pip
    echo "18.6. Instalar Vim.............."; apt-get install vim
    echo "18.7. Instalar Nano............."; apt-get install nano
    echo "18.8. Instalar pear............."; apt-get install php-pear
    echo "18.9. Instalar PHPUnit..........";
    pear config-set auto_discover 1
    mv phpunit-patched /usr/share/phpunit
    echo include_path = ".:/usr/share/phpunit:/usr/share/phpunit/PHPUnit" >> /etc/php5/apache2/php.ini
    echo include_path = ".:/usr/share/phpunit:/usr/share/phpunit/PHPUnit" >> /etc/php5/cli/php.ini
    service apache2 restart
    say_done
}

# 19. Tunear y Asegurar Kernel
function tunning_kernel() {
    write_title "19. Tunear Kernel"
    cp templates/sysctl.conf /etc/sysctl.conf; echo " OK"
    cp templates/ufw /etc/default/ufw
    sysctl -e -p
    say_done
}

# 20. Instalar RootKit Hunter
function install_rootkithunter() {
    write_title "20. Instalar Rootkit Hunter"
    cd rkhunter-1.4.2/
    sh installer.sh --layout /usr --install
    cd ..
    rkhunter --update
    rkhunter --propupd
    echo " *** Para correr RootKit Hunter solo debe ejecutar ***"
    echo "     rkhunter -c --enable all --disable none"
    echo "     Puede ver el reporte detallado en /var/log/rkhunter.log"
    say_done
}

# 21. Tunnear el archivo .bashrc
function tunning_bashrc() {
    write_title "21. Reemplazar .bashrc"
    cp templates/bashrc-root /root/.bashrc
    cp templates/bashrc-user /home/$username/.bashrc
    chown $username:$username /home/$username/.bashrc
    say_done
}


# 22. Tunnear Vim
function tunning_vim() {
    write_title "22. Tunnear Vim"
    tunning vimrc
}


# 23. Tunnear Nano
function tunning_nano() {
    write_title "23. Tunnear Nano"
    tunning nanorc
}


# 24. Agregar tarea de actualización diaria
function add_updating_task() {
    write_title "24. Agregar tarea de actualización diaria al Cron"
    tarea="@daily apt-get update; apt-get dist-upgrade -y"
    touch tareas
    echo $tarea >> tareas
    crontab tareas
    rm tareas
    say_done
}


# 25. Agregar comandos personalizados
function add_commands() {
    write_title "25. Agregar comandos personalizados"
    add_command_blockip     # Agregar regla bloqueo a iptables
    say_done
}


# 26. Instalar PortSentry
function install_portsentry() {
    write_title "# 26. Instalar y configurar el antiscan de puertos PortSentry"
    apt-get install portsentry
    mv /etc/portsentry/portsentry.conf /etc/portsentry/portsentry.conf-original
    cp templates/portsentry /etc/portsentry/portsentry.conf
    sed s/tcp/atcp/g /etc/default/portsentry > salida.tmp
    mv salida.tmp /etc/default/portsentry
    /etc/init.d/portsentry restart
    say_done
}

# 27. Otros pasos de Seguridad
function extrasec_steps() {
    write_title "# 27. Asegurando tty, y conf grub"
    echo tty1 > /etc/securetty
    chmod 700 /root
    chmod 600 /boot/grub/grub.cfg
    #Proteger contra IP Spoofing
    echo nospoof on >> /etc/host.conf
    #Desinstalar AT y Restringiendo Cron a Root
    apt-get purge at
    echo " Asegurando Cron "
    touch /etc/cron.allow
    chmod 600 /etc/cron.allow
    awk -F: '{print $1}' /etc/passwd | grep -v root > /etc/cron.deny
    say_done
}

# 28. Instalar Unhide
function install_unhide() {
    write_title "#28. Instalando Unhide"
    apt-get -y install unhide
    echo " Unhide es una Herramienta para Detectar Procesos Ocultos "
    echo " Para más información sobre la Herramienta ver los manpages "
    echo " man unhide "
    say_done
}

# 29. Instalar Tiger
#Tiger es un sistema de Auditorías y Detección de Intrusos
function install_tiger() {
    write_title " #29. Instalando Tiger "
    apt-get -y install tiger
    echo " Para más información sobre la Herramienta, ver los manpages "
    echo " man tiger "
    say_done
}


#30. Deshabilitar Compiladores
function disable_compilers() {
    write_title " 30. Deshabilitando Compiladores "
    chmod 000 /usr/bin/byacc
    chmod 000 /usr/bin/yacc
    chmod 000 /usr/bin/bcc
    chmod 000 /usr/bin/kgcc
    chmod 000 /usr/bin/cc
    chmod 000 /usr/bin/gcc
    chmod 000 /usr/bin/*c++
    chmod 000 /usr/bin/*g++
    echo " Si desea utilizarlo mas adelante solo debe cambiar los permisos chmod 755"
    echo " Ejemplo: chmod 755 /usr/bin/gcc "
    say_done
}

#31. Restringir Acceso Archivos Conf. Apache
function restrict_apacheconf() {
     write_title " 31. Restringiendo acceso Archivos Conf. Apache "
     chmod 750 /etc/apache2/conf*
     chmod 511 /usr/bin/apache2
     chmod 750 /var/log/apache2/
     chmod 640 /etc/apache2/conf-available/*
     chmod 640 /etc/apache2/conf-enabled/*
     chmod 640 /etc/apache2/apache2.conf
     say_done
}

# 32. Reiniciar servidor
function final_step() {
    write_title "32. Finalizar deploy"
    replace USERNAME $username SERVERIP $serverip < templates/texts/bye
    echo -n " ¿Ha podido conectarse por SHH como $username? (y/n) "
    read respuesta
    if [ "$respuesta" == "y" ]; then
        reboot
    else
        echo "El servidor NO será reiniciado y su conexión permanecerá abierta."
        echo "Bye."
    fi
}


is_root_user                    #  0. Verificar si es usuario root o no
set_hostname                    #  1. Configurar Hostname
set_hour                        #  2. Configurar zona horaria
sysupdate                       #  3. Actualizar el sistema
set_new_user                    #  4. Crear un nuevo usuario con privilegios
give_instructions               #  5. Instrucciones para generar una RSA Key
move_rsa                        #  6. Mover la llave pública RSA generada
ssh_reconfigure                 #  7. Securizar SSH
set_iptables_rules              #  8. Establecer reglas para iptables
install_fail2ban                #  9. Instalar fail2ban
install_mysql                   # 10. Instalar, Configurar y Optimizar MySQL
install_php                     # 11. Instalar, configurar y optimizar PHP
install_modsecurity             # 12. Instalar ModSecurity
install_owasp_core_rule_set     # 13. Instalar OWASP para ModSecuity
configure_apache                # 14. Configurar y optimizar Apache
install_modevasive              # 15. Instalar ModEvasive
install_modqosspam              # 16. Instalar Mod_qos/spamhaus
config_fail2ban                 # 17. Configurar fail2ban
install_aditional_packages      # 18. Instalación de paquetes adicionales
tunning_kernel                  # 19. Asegurar Kernel
install_rootkithunter           # 20. Instalar RootKitHunter
tunning_bashrc                  # 21. Tunnear el archivo .bashrc
tunning_vim                     # 22. Tunnear Vim
tunning_nano                    # 23. Tunnear Nano
add_updating_task               # 24. Agregar tarea de actualización diaria
add_commands                    # 25. Agregar comandos personalizados
install_portsentry              # 26. Instalar PortSentry
extrasec_steps                  # 27. Otros pasos de Seguridad
install_unhide                  # 28. Instalar Unhide
install_tiger                   # 29. Instalar Tiger
disable_compilers               # 30. Deshabilitar Compiladores
restrict_apacheconf             # 31. Restringir Acceso Archivos Conf. Apache
final_step                      # 32. Reiniciar servidor

